#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# mq_install::software
#
# @summary This class install's specified version of the the MQ software on the system.
# The source zip file from IBM is found at the location specified at `source_location`. By default it will install only the minimal required set. By using the `install_...` parameters, you can install optional components.
# 
# Here is an example on how to use it:
# 
# ``` puppet
# class { '::mq_install::software':
#   source_location  => '/software',
#   version          => '9.0.0.0'
#   tmp_dir          => '/tmp'
#   install_ams      => true,
#   install_ft       => false,
#   install_client   => false,
#   install_explorer => true,
#   install_xrserver => false,
#   install_samples  => false,
#   install_sdk      => false,
#   install_man      => false,
#   install_gskit    => false,
#   install_amqp     => false,
#   logoutput        => 'true,
# }
# ```
# 
# On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [Boolean] install_ams
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ams => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_ft
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ft => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_client
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_client => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_explorer
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_explorer => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_xrserver
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_xserver => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_samples
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_samples => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_sdk
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_sdk => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_man
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_man => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_gskit
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_gskit => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_amqp
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_amqp => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_web
#    
#
# @param [Boolean] install_salesforce
#    Specify a value of `true` when you want to install the Bridge to Salesforce component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_salesforce => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to Salesforce and IBM MQ, then run the runmqsfb command to subscribe to events from Salesforce and publish them to an IBM MQ network.
#
# @param [Boolean] install_blockchain
#    Specify a value of `true` when you want to install the IBM MQ Bridge to blockchain component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_blockchain => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to your blockchain network and IBM MQ. You can then run the runmqbcb command to start the bridge and send queries and updates to, and receive responses from your blockchain network. The queue manager that the bridge connects to must be an IBM MQ Advanced queue manager.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#
# @param [Boolean] install_rdqm
#    Specify a value of `true` when you want to install the replicated data queue manager component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_rqdm => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to make the replicated data queue manager high availability configuration available. See Installing RDQM (replicated data queue managers) for more information.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          logoutput => true,
#          ...
#        }
#
#--++--
class mq_install::software(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                           # Version of the MQ software to install
  String                               $tmp_dir            = '/tmp',          # Directory used for unpacking
  Boolean                              $install_ams        = false,           # Install AMS packages
  Boolean                              $install_ft         = false,           # Install FT packages
  Boolean                              $install_client     = false,           # Install Client packages
  Boolean                              $install_explorer   = false,           # Install Explorer packages
  Boolean                              $install_xrserver   = false,           # Install XRServer packages
  Boolean                              $install_samples    = false,           # Install Samples packages
  Boolean                              $install_sdk        = false,           # Install SDK packages
  Boolean                              $install_man        = false,           # Install man packages
  Boolean                              $install_gskit      = false,           # Install GSKit packages
  Boolean                              $install_amqp       = false,           # Install AMQP packages
  Boolean                              $install_web        = false,           # REST API and Console	
  Boolean                              $install_salesforce = false,           # IBM MQ Bridge to Salesforce
  Boolean                              $install_blockchain = false,           # IBM MQ Bridge to blockchain
  Boolean                              $install_rdqm       = false,           # replicated data queue manage
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  mq_install { 'setmq_install::software': }
  #
  # Setup some variables
  #
  $supported_major_versions = ['8','9']    # At this point in time only versio 8 is tested. Probably other versions work as well.
  $version_components       = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $major_version            = $version_components[0]
  $package_version          = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  unless $major_version in $supported_major_versions {
    fail "MQ major version ${major_version} not (yet) supported. Only versions ${supported_major_versions.join(',')} supported."
  }

  case $major_version {
    '8':     { $file_name = "WS_MQ_V${version}_LINUX_ON_X86_64_IM" }
    '9':     { $file_name = "IBM_MQ_${version}_LINUX_X86-64"}
    default: { fail 'internal error. Unknown major version found.'}
  }

  #
  # If the installed version is the same as the requested version. Do nothing
  #
  if $version != $::mq_version {

    #
    # When the tmp_dir is not defined, define it here.
    #
    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
        before => Archive["${tmp_dir}/${file_name}.tar.gz"],
      }
    }

    archive { "${tmp_dir}/${file_name}.tar.gz":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}/${file_name}.tar.gz",
      creates      => "${tmp_dir}/MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }

    -> exec { 'Accept MQ License Requirements':
      command => "${tmp_dir}/MQServer/mqlicense.sh -accept",
      creates => "/tmp/mq_license_${version}/license/status.dat"
    }

    #
    # Create array of selected packages. First parameter is the set of base packages,
    # They must always be selected.
    # Second parameter is an array of selections. The third parameter is an array of packages,
    # belonging to the selection.
    #
    $packages = mq_install::selected_packages(
      [
        'MQSeriesRuntime',
        'MQSeriesJRE',
        'MQSeriesJava',
        'MQSeriesServer',
      ],[
        $install_ams,
        $install_ft,
        $install_client,
        $install_explorer,
        $install_xrserver,
        $install_samples,
        $install_sdk,
        $install_man,
        $install_gskit,
        $install_amqp,
        $install_web,
        $install_salesforce,
        $install_blockchain,
        $install_rdqm,
      ],[
        'MQSeriesAMS',
        ['MQSeriesFTBase', 'MQSeriesFTAgent','MQSeriesFTLogger','MQSeriesFTService','MQSeriesFTTools'],
        'MQSeriesClient',
        'MQSeriesExplorer',
        'MQSeriesXRService',
        'MQSeriesSamples',
        'MQSeriesSDK',
        'MQSeriesMan',
        'MQSeriesGSKit',
        'MQSeriesAMQP',
        'MQSeriesWeb',
        'MQSeriesSFBridge',
        'MQSeriesBCBridge',
        'MQSeriesRDQM',
      ])

    $packages.each | $package| {
      package {$package:
        ensure   => $package_version,
        provider => 'rpm',
        source   => "${tmp_dir}/MQServer/${package}-${package_version}.x86_64.rpm",
        before   =>  File['Remove temporary files'],
        require  =>  Exec['Accept MQ License Requirements'],
      }
    }

    file {'Remove temporary files':
      ensure  => 'absent',
      path    => "${tmp_dir}/MQServer",
      recurse => true,
      force   => true,
    }
  }
}
